/*
 * Show concatenated filesystem object contents using the client library.
 *
 * This should eventually use C library functions instead of client library
 * functions.
 *
 * Copyright (C) 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <stdio.h>



/* NOTE: For inclusion in the C library: stream acquisition and access. */

#include <fsclient/client.h>
#include <systypes/env.h>
#include <systypes/fcntl.h>

file_t *output;



/* Transfer size for communication. */

static const offset_t TO_TRANSFER = 1024;



/* Show an object from the filesystem. */

static int show_object(const char *path)
{
    char *buffer[TO_TRANSFER];
    file_t *file = client_open(path, O_RDONLY);
    offset_t nread;

    if (!client_opened(file))
        return 1;

    while ((nread = client_read(file, buffer, TO_TRANSFER)))
    {
        if (!client_write(output, buffer, nread))
            break;
    }

    client_close(file);

    return 0;
}

/* List objects in the filesystem image. */

int main(int argc, char *argv[])
{
    int i;

    /* NOTE: For inclusion in the C library: stream acquisition and access. */

    output = client_get_stream(ENV_OUTPUT_STREAM_NAME, O_WRONLY);

    /* Show all specified objects. */

    for (i = 1; i < argc; i++)
    {
        if (show_object(argv[i]))
            return 1;
    }

    /* NOTE: For inclusion in the C library: stream acquisition and access. */

    client_flush(output);

    return 0;
}

/* vim: tabstop=4 expandtab shiftwidth=4
*/
